#ifndef RENDERSYSTEMS_GLES2_INCLUDE_OGREGLES2CONFIG_H_
#define RENDERSYSTEMS_GLES2_INCLUDE_OGREGLES2CONFIG_H_

#define OGRE_NO_GLES2_CG_SUPPORT 1

#define OGRE_NO_GLES2_GLSL_OPTIMISER 1

#define OGRE_NO_GLES3_SUPPORT 1

#endif
